@@ -1,5 +1,5 @@
 #
-#   Copyright 2007-2016 The OpenMx Project
+#   Copyright 2007-2017 The OpenMx Project
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -125,7 +125,7 @@ twoFactorModel <- mxModel("Two Factor Model Matrix Specification",
 twoFactorFit <- mxRun(twoFactorModel)
 
 summary(twoFactorFit)
-twoFactorFit$output$estimate
+coef(twoFactorFit)
 
 
 omxCheckCloseEnough(twoFactorFit$output$estimate[["l2"]], 0.9723, 0.01)
@@ -148,4 +148,4 @@ omxCheckCloseEnough(twoFactorFit$output$estimate[["meanx4"]], 2.9554, 0.01)
 omxCheckCloseEnough(twoFactorFit$output$estimate[["meanx5"]], 2.9562, 0.01)
 omxCheckCloseEnough(twoFactorFit$output$estimate[["meanx6"]], 2.9673, 0.01)
 # Compare OpenMx results to Mx results 
-# -----------------------------------------------------------------------------
\ No newline at end of file
+# -----------------------------------------------------------------------------
