@@ -378,3 +378,42 @@ if(mxOption(NULL,"Default optimizer") %in% c("NPSOL","SLSQP")){
 	colnames(tbl) <- c("Gradient?","Jacobians?","Fitfunction evaluations")
 	print(tbl)
 }
+
+
+if(mxOption(NULL,"Default optimizer") == "SLSQP"){
+	#With GDsearch, Nelder-Mead can get a good solution even though none of its initial vertices is feasible,
+	#but it has to be held to a strict feasibility tolerance:
+	foo <- mxComputeNelderMead(
+		iniSimplexType="smartRight", xTolProx=1e-12, fTolProx=1e-8, eqConstraintMthd="GDsearch", 
+		nudgeZeroStarts=F)
+	plan <- omxDefaultComputePlan()
+	plan$steps <- list(foo, plan$steps$RE)
+	nmpowell <- mxModel(
+		"PowellBenchmarkWithJacobians",
+		plan,
+		mxMatrix(type="Full",nrow=1,ncol=5,free=T,values=c(-2,2,2,-1,-1),labels=paste("x",1:5,sep=""),name="X"),
+		mxAlgebra( exp(prod(X)), name="powellfunc"),
+		mxAlgebra( cbind(powellfunc*X[1,2]*X[1,3]*X[1,4]*X[1,5],
+										 powellfunc*X[1,1]*X[1,3]*X[1,4]*X[1,5],
+										 powellfunc*X[1,1]*X[1,2]*X[1,4]*X[1,5],
+										 powellfunc*X[1,1]*X[1,2]*X[1,3]*X[1,5],
+										 powellfunc*X[1,1]*X[1,2]*X[1,3]*X[1,4]), 
+							 name="objgrad", 
+							 dimnames=list(NULL,paste("x",1:5,sep="")) ),
+		#Nelder-Mead benefits from analytic Jacobians if using eqConstraintMthd="GDsearch":
+		mxConstraint(sum(X%^%2) - 10 == 0, name="c1",jac="jac1" ),
+		mxConstraint(X[1,2]*X[1,3]-5*X[1,4]*X[1,5] == 0, name="c2",jac="jac2" ),
+		mxConstraint(X[1,1]^3 + X[1,2]^3 + 1 == 0, name="c3",jac="jac3" ),
+		mxAlgebra(cbind(2*X[1,1],2*X[1,2],2*X[1,3],2*X[1,4],2*X[1,5]),name="jac1",
+							dimnames=list(NULL,paste("x",1:5,sep=""))
+		),
+		mxAlgebra(cbind(0,X[1,3],X[1,2],-5*X[1,5],-5*X[1,4]),name="jac2",
+							dimnames=list(NULL,paste("x",1:5,sep=""))),
+		mxAlgebra(cbind(3*X[1,1]^2, 3*X[1,2]^2, 0, 0, 0),name="jac3",dimnames=list(NULL,paste("x",1:5,sep=""))),
+		mxFitFunctionAlgebra(algebra="powellfunc",gradient="objgrad")
+	)
+	nmpowell <- mxOption(nmpowell,"Feasibility tolerance",0.001)
+	nmprun <- mxRun(nmpowell)
+	nmprun$compute$steps[[1]]$output$constraintFunctionValues
+	omxCheckCloseEnough(coef(powellrun2),coef(nmprun),0.02)
+}
