@@ -28,9 +28,8 @@ factorModel <- mxModel("One Factor", type="RAM",
     mxPath(from=latents, to=manifests, free=c(FALSE,TRUE,TRUE,TRUE,TRUE), values=1),
     mxPath(from=manifests, arrows=2, lbound=.0001),
     mxPath(from=latents, arrows=2, free=TRUE, values=1.0),
-    mxData(cov(demoOneFactor), type="cov", numObs=500),
-    # mxPath(from="one", to=manifests, arrows=1, free=T, values=colMeans(demoOneFactor)),
-    # mxData(demoOneFactor, type="raw"),
+    mxPath(from="one", to=manifests, arrows=1, free=T, values=colMeans(demoOneFactor)),
+    mxData(demoOneFactor, type="raw"),
     mxMatrix("Iden", nrow=nVars, name="I"),
     mxMatrix("Full", free=FALSE, values=diag(nrow=nManifest, ncol=nVars), name="Eff"),
     mxAlgebra(Eff%*%solve(I-A), name="Z"),
@@ -39,7 +38,11 @@ factorModel <- mxModel("One Factor", type="RAM",
     mxCI(c("P"))
 )
 factorFit <- mxRun(factorModel, intervals=FALSE)
-omxCheckCloseEnough(factorFit$output$fit, -3660.596, .01)
+omxCheckCloseEnough(factorFit$output$fit, 934.095, .01)
+
+factorBoot <- mxBootstrap(factorFit, 100L, OK=c("OK","OK/green", "nonzero gradient"))
+omxCheckError(mxBootstrapEval(P, factorFit, compute=T),
+	      "Compute plan MxComputeSequence found in model 'One Factor' instead of MxComputeBootstrap. Have you run this model through mxBootstrap already?")
 
 if (mxOption(NULL, 'Default optimizer') != "SLSQP") {ctype = 'none'} else {ctype = 'ineq'}
 
@@ -47,6 +50,14 @@ factorFitCI <- mxRun(mxModel(factorFit, mxComputeConfidenceInterval(plan=mxCompu
 factorSummCI <- summary(factorFitCI)
 summary(factorFitCI)
 
+set.seed(42L)
+bci <- mxBootstrapEval(P, factorBoot, bq=c(.025,.975))
+print(bci)
+omxCheckCloseEnough(factorFitCI$output$confidenceIntervals[,'lbound'] - bci[,"2.5%"],
+                    rep(0,5), .05)
+omxCheckCloseEnough(factorFitCI$output$confidenceIntervals[,'ubound'] - bci[,"97.5%"],
+                    rep(0,5), .02)
+
 omxCheckCloseEnough(coef(factorFit), coef(factorFitCI))
 omxCheckCloseEnough(factorFit$output$fit, factorFitCI$output$fit, 0)
 omxCheckCloseEnough(mxEval(Z, factorFit), mxEval(Z, factorFitCI))
