@@ -19,7 +19,7 @@
 #include <algorithm>
 #include <stdarg.h>
 #include <limits>
-
+#include <random>
 //#include <iostream>
 
 #include "glue.h"
@@ -36,6 +36,24 @@
 
 void pda(const double *ar, int rows, int cols);
 
+SEXP allocInformVector(int size)
+{
+	const char *statusCodeLabels[] = { // see ComputeInform type
+		"OK", "OK/green",
+		"infeasible linear constraint",
+		"infeasible non-linear constraint",
+		"iteration limit",
+		"not convex",
+		"nonzero gradient",
+		"bad deriv",
+		"?",
+		"internal error",
+		"infeasible start"
+	};
+	return makeFactor(Rf_allocVector(INTSXP, size),
+			  OMX_STATIC_ARRAY_SIZE(statusCodeLabels), statusCodeLabels);
+}
+
 void FitContext::queue(HessianBlock *hb)
 {
 	if (hb->vars.size() == 0) {
@@ -1521,9 +1539,8 @@ class omxComputeOnce : public omxCompute {
 
 class ComputeEM : public omxCompute {
 	typedef omxCompute super;
-	std::vector< omxExpectation* > expectations;
-	const char *predict;
-	omxCompute *fit1;  // maybe rename to mstep TODO
+	omxCompute *estep;
+	omxCompute *mstep;
 	omxMatrix *fit3;   // rename to observedFit
 	int EMcycles;
 	int maxIter;
@@ -1569,7 +1586,6 @@ class ComputeEM : public omxCompute {
 	size_t maxHistLen;
 	int semProbeCount;
 
-	void setExpectationPrediction(FitContext *fc, const char *context);
 	void observedFit(FitContext *fc);
 	template <typename T1>
 	void accelLineSearch(bool major, FitContext *fc, Eigen::MatrixBase<T1> &preAccel);
@@ -1623,6 +1639,38 @@ class ComputeReportExpectation : public omxCompute {
         virtual void reportResults(FitContext *fc, MxRList *slots, MxRList *out);
 };
 
+class ComputeBootstrap : public omxCompute {
+	typedef omxCompute super;
+	
+	struct context {
+		omxData *data;
+		double *origRowWeights;
+		std::vector<double> origCumSum;
+		std::vector<double> resample;
+	};
+	std::vector< context > contexts;
+	omxCompute *plan;
+	int verbose;
+	int numReplications;
+	int seed;
+	//std::vector<double> quantile;
+	bool parallel;
+	int only;
+
+	int previousNumParam;
+	SEXP previousData;
+	SEXP rawOutput;
+	MxRList onlyWeight;
+
+ public:
+	ComputeBootstrap() : plan(0) {};
+	virtual ~ComputeBootstrap();
+	virtual void initFromFrontend(omxState *, SEXP rObj);
+	virtual void computeImpl(FitContext *fc);
+	virtual void collectResults(FitContext *fc, LocalComputeResult *lcr, MxRList *out);
+	virtual void reportResults(FitContext *fc, MxRList *, MxRList *result);
+};
+
 static class omxCompute *newComputeSequence()
 { return new omxComputeSequence(); }
 
@@ -1647,6 +1695,9 @@ static class omxCompute *newComputeReportDeriv()
 static class omxCompute *newComputeReportExpectation()
 { return new ComputeReportExpectation(); }
 
+static class omxCompute *newComputeBootstrap()
+{ return new ComputeBootstrap(); }
+
 struct omxComputeTableEntry {
         char name[32];
         omxCompute *(*ctor)();
@@ -1666,7 +1717,8 @@ static const struct omxComputeTableEntry omxComputeTable[] = {
 	{"MxComputeConfidenceInterval", &newComputeConfidenceInterval},
 	{"MxComputeReportExpectation", &newComputeReportExpectation},
 	{"MxComputeTryHard", &newComputeTryHard},
-	{"MxComputeNelderMead", &newComputeNelderMead}
+	{"MxComputeNelderMead", &newComputeNelderMead},
+	{"MxComputeBootstrap", &newComputeBootstrap},
 };
 
 omxCompute *omxNewCompute(omxState* os, const char *type)
@@ -1682,7 +1734,7 @@ omxCompute *omxNewCompute(omxState* os, const char *type)
                 }
         }
 
-        if (!got) Rf_error("Compute %s is not implemented", type);
+        if (!got) Rf_error("Compute plan step '%s' is not implemented", type);
 
         return got;
 }
@@ -1853,27 +1905,15 @@ void ComputeEM::initFromFrontend(omxState *globalState, SEXP rObj)
 
 	super::initFromFrontend(globalState, rObj);
 
-	Rf_protect(slotValue = R_do_slot(rObj, Rf_install("expectation")));
-	for (int wx=0; wx < Rf_length(slotValue); ++wx) {
-		int objNum = INTEGER(slotValue)[wx];
-		omxExpectation *expectation = globalState->expectationList[objNum];
-		omxCompleteExpectation(expectation);
-		expectations.push_back(expectation);
-	}
-
-	Rf_protect(slotValue = R_do_slot(rObj, Rf_install("predict")));
-	{
-		// Should accept a vector here TODO
-		if (Rf_length(slotValue) != 1) Rf_error("Not implemented");
-		SEXP elem;
-		Rf_protect(elem = STRING_ELT(slotValue, 0));
-		predict = CHAR(elem);
-	}
+	Rf_protect(slotValue = R_do_slot(rObj, Rf_install("estep")));
+	Rf_protect(s4class = STRING_ELT(Rf_getAttrib(slotValue, R_ClassSymbol), 0));
+	estep = omxNewCompute(globalState, CHAR(s4class));
+	estep->initFromFrontend(globalState, slotValue);
 
 	Rf_protect(slotValue = R_do_slot(rObj, Rf_install("mstep")));
 	Rf_protect(s4class = STRING_ELT(Rf_getAttrib(slotValue, R_ClassSymbol), 0));
-	fit1 = omxNewCompute(globalState, CHAR(s4class));
-	fit1->initFromFrontend(globalState, slotValue);
+	mstep = omxNewCompute(globalState, CHAR(s4class));
+	mstep->initFromFrontend(globalState, slotValue);
 
 	Rf_protect(slotValue = R_do_slot(rObj, Rf_install("observedFit")));
 	fit3 = globalState->algebraList[ INTEGER(slotValue)[0] ];
@@ -2037,15 +2077,6 @@ void ComputeEM::initFromFrontend(omxState *globalState, SEXP rObj)
 	origEigenvalues = NULL;
 }
 
-void ComputeEM::setExpectationPrediction(FitContext *fc, const char *context)
-{
-	for (size_t wx=0; wx < expectations.size(); ++wx) {
-		omxExpectation *expectation = expectations[wx];
-		if (verbose >= 4) mxLog("ComputeEM: expectation[%d] %s predict %s", (int) wx, expectation->name, context);
-		omxExpectationCompute(fc, expectation, context);
-	}
-}
-
 template <typename T>
 bool ComputeEM::probeEM(FitContext *fc, int vx, double offset, Eigen::MatrixBase<T> &rijWork)
 {
@@ -2060,15 +2091,15 @@ bool ComputeEM::probeEM(FitContext *fc, int vx, double offset, Eigen::MatrixBase
 	if (verbose >= 3) mxLog("ComputeEM: probe %d of %s offset %.6f",
 				1+paramProbeCount[vx], fc->varGroup->vars[vx]->name, offset);
 
-	setExpectationPrediction(fc, predict);
+	estep->compute(fc);
+	fc->wanted &= ~FF_COMPUTE_HESSIAN;  // discard garbage
 	int informSave = fc->getInform();  // not sure if we want to hide inform here TODO
-	fit1->compute(fc);
+	mstep->compute(fc);
 	if (fc->getInform() > INFORM_UNCONVERGED_OPTIMUM) {
 		if (verbose >= 3) mxLog("ComputeEM: probe failed with code %d", fc->getInform());
 		failed = true;
 	}
 	fc->setInform(informSave);
-	setExpectationPrediction(fc, "nothing");
 
 	rijWork.col(paramProbeCount[vx]) = (Est - optimum) / offset;
 
@@ -2158,13 +2189,14 @@ void ComputeEM::computeImpl(FitContext *fc)
 		++ EMcycles;
 		memcpy(&prevEst[0], fc->est, sizeof(double) * fc->numParam);
 		if (verbose >= 4) mxLog("ComputeEM[%d]: E-step", EMcycles);
-		setExpectationPrediction(fc, predict);
+		estep->compute(fc);
+		fc->wanted &= ~FF_COMPUTE_DERIV;
 
 		{
 			if (verbose >= 4) mxLog("ComputeEM[%d]: M-step", EMcycles);
-			FitContext *fc1 = new FitContext(fc, fit1->varGroup);
+			FitContext *fc1 = new FitContext(fc, mstep->varGroup);
 			int startIter = fc1->iterations;
-			fit1->compute(fc1);
+			mstep->compute(fc1);
 			fc1->wanted &= ~FF_COMPUTE_HESSIAN;  // discard garbage
 			mstepIter = fc1->iterations - startIter;
 			totalMstepIter += mstepIter;
@@ -2172,7 +2204,6 @@ void ComputeEM::computeImpl(FitContext *fc)
 			fc1->updateParentAndFree();
 		}
 
-		setExpectationPrediction(fc, "nothing");
 		if (accel) {
 			if (!lbound.size()) {
 				// bounds might have changed
@@ -2206,6 +2237,10 @@ void ComputeEM::computeImpl(FitContext *fc)
 			observedFit(fc);
 		}
 
+		if (!std::isfinite(fc->fit)) {
+			omxRaiseErrorf("%s: fit not finite in iteration %d", name, EMcycles);
+		}
+
 		double change = 0;
 		if (prevFit != 0) {
 			if (verbose >= 5) {
@@ -2298,7 +2333,7 @@ void ComputeEM::dEstep(FitContext *fc, Eigen::MatrixBase<T1> &x, Eigen::MatrixBa
 		omxFitFunctionCompute(infoFitFunction[fx]->fitFunction, FF_COMPUTE_GRADIENT, fc);
 	}
 	result = fc->grad;
-	Global->reportProgress("MxComputeEM", fc);
+	reportProgress(fc);
 }
 
 void ComputeEM::Oakes(FitContext *fc)
@@ -2308,7 +2343,9 @@ void ComputeEM::Oakes(FitContext *fc)
 	int wanted = fc->wanted;
 	const int freeVars = (int) fc->varGroup->vars.size();
 
-	setExpectationPrediction(fc, predict);
+	estep->compute(fc);
+	fc->wanted &= ~FF_COMPUTE_HESSIAN;  // discard garbage
+
 	fc->grad = Eigen::VectorXd::Zero(fc->numParam);
 	for (size_t fx=0; fx < infoFitFunction.size(); ++fx) {
 		omxFitFunctionCompute(infoFitFunction[fx]->fitFunction, FF_COMPUTE_PREOPTIMIZE, fc);
@@ -2322,7 +2359,7 @@ void ComputeEM::Oakes(FitContext *fc)
 
 	Eigen::MatrixXd jacobian(freeVars, freeVars);
 	estep_jacobian_functional ejf(this, fc);
-	fd_jacobian(GradientAlgorithm_Forward, 1, 1e-5, ejf, refGrad, optimumCopy, jacobian);
+	fd_jacobian<false>(GradientAlgorithm_Forward, 1, 1e-5, ejf, refGrad, optimumCopy, jacobian);
 
 	fc->infoMethod = infoMethod;
 	fc->preInfo();
@@ -2330,7 +2367,6 @@ void ComputeEM::Oakes(FitContext *fc)
 		omxFitFunctionCompute(infoFitFunction[fx]->fitFunction, FF_COMPUTE_INFO, fc);
 	}
 	fc->postInfo();
-	setExpectationPrediction(fc, "nothing");
 
 	fc->refreshDenseHess();
 	double *hess = fc->getDenseHessUninitialized();
@@ -2470,7 +2506,9 @@ void ComputeEM::MengRubinFamily(FitContext *fc)
 	//mxLog("rij symm");
 	//pda(rij.data(), freeVars, freeVars);
 
-	setExpectationPrediction(fc, predict);
+	estep->compute(fc);
+	fc->wanted &= ~FF_COMPUTE_HESSIAN;  // discard garbage
+
 	int wanted = fc->wanted;
 	fc->wanted = 0;
 	fc->infoMethod = infoMethod;
@@ -2479,7 +2517,6 @@ void ComputeEM::MengRubinFamily(FitContext *fc)
 		omxFitFunctionCompute(infoFitFunction[fx]->fitFunction, FF_COMPUTE_INFO, fc);
 	}
 	fc->postInfo();
-	setExpectationPrediction(fc, "nothing");
 
 	Rf_protect(inputInfoMatrix = Rf_allocMatrix(REALSXP, freeVars, freeVars));
 	double *hess = REAL(inputInfoMatrix);
@@ -2532,7 +2569,7 @@ void ComputeEM::collectResults(FitContext *fc, LocalComputeResult *lcr, MxRList
 	super::collectResults(fc, lcr, out);
 
 	std::vector< omxCompute* > clist(1);
-	clist[0] = fit1;
+	clist[0] = mstep;
 
 	collectResultsHelper(fc, clist, lcr, out);
 }
@@ -2586,7 +2623,8 @@ ComputeEM::~ComputeEM()
 {
 	if (accel) delete accel;
 
-	delete fit1;
+	delete estep;
+	delete mstep;
 
 	for (size_t hx=0; hx < estHistory.size(); ++hx) {
 		delete [] estHistory[hx];
@@ -2622,8 +2660,9 @@ void omxComputeOnce::initFromFrontend(omxState *globalState, SEXP rObj)
 		int objNum = INTEGER(slotValue)[wx];
 		if (objNum >= 0) {
 			omxMatrix *algebra = globalState->algebraList[objNum];
-			if (!algebra->fitFunction) Rf_error("ComputeOnce can only handle fit functions");
-			omxCompleteFitFunction(algebra);
+			if (algebra->fitFunction) {
+				omxCompleteFitFunction(algebra);
+			}
 			algebras.push_back(algebra);
 		} else {
 			omxExpectation *expectation = globalState->expectationList[~objNum];
@@ -2642,6 +2681,9 @@ void omxComputeOnce::initFromFrontend(omxState *globalState, SEXP rObj)
 	Rf_protect(slotValue = R_do_slot(rObj, Rf_install("what")));
 	int whatLen = Rf_length(slotValue);
 	if (algebras.size()) {
+		if (whatLen == 0) {
+			fit = true;
+		}
 		for (int wx=0; wx < whatLen; ++wx) {
 			SEXP elem;
 			Rf_protect(elem = STRING_ELT(slotValue, wx));
@@ -2699,15 +2741,10 @@ void omxComputeOnce::initFromFrontend(omxState *globalState, SEXP rObj)
 
 	for (int ax=0; ax < (int) algebras.size(); ++ax) {
 		omxFitFunction *ff = algebras[ax]->fitFunction;
-		if (!ff) {
-			Rf_error("ComputeOnce can only handle fit functions");
-			continue;
-		}
-
-		if (gradient && !ff->gradientAvailable) {
+		if (gradient && (!ff || !ff->gradientAvailable)) {
 			Rf_error("Gradient requested but not available");
 		}
-		if ((hessian || ihessian || hgprod) && !ff->hessianAvailable) {
+		if ((hessian || ihessian || hgprod) && (!ff || !ff->hessianAvailable)) {
 			// add a separate flag for hgprod TODO
 			Rf_error("Hessian requested but not available");
 		}
@@ -2754,17 +2791,24 @@ void omxComputeOnce::computeImpl(FitContext *fc)
 
 		for (size_t wx=0; wx < algebras.size(); ++wx) {
 			omxMatrix *algebra = algebras[wx];
-			omxAlgebraPreeval(algebra, fc);
-			ComputeFit("Once", algebra, want, fc);
-			if (infoMat) {
-				fc->postInfo();
+			if (algebra->fitFunction) {
+				omxAlgebraPreeval(algebra, fc);
+				ComputeFit("Once", algebra, want, fc);
+				if (infoMat) {
+					fc->postInfo();
+				}
+			} else {
+				omxMarkDirty(algebra);
+				omxRecompute(algebra, fc);
 			}
 		}
 	} else if (expectations.size()) {
 		if (predict.size() > 1) Rf_error("Not implemented");
+		const char *pr1 = "nothing"; // better to default to 0 ?
+		if (predict.size()) pr1 = predict[0];
 		for (size_t wx=0; wx < expectations.size(); ++wx) {
 			omxExpectation *expectation = expectations[wx];
-			omxExpectationCompute(fc, expectation, predict[0], how);
+			omxExpectationCompute(fc, expectation, pr1, how);
 		}
 	}
 }
@@ -2780,6 +2824,8 @@ void omxComputeOnce::reportResults(FitContext *fc, MxRList *slots, MxRList *out)
 
 void ComputeStandardError::reportResults(FitContext *fc, MxRList *slots, MxRList *)
 {
+	if (!(fc->wanted & (FF_COMPUTE_HESSIAN | FF_COMPUTE_IHESSIAN))) return;
+
 	fc->allocStderrs();  // at least report NAs
 
 	const size_t numParams = fc->numParam;
@@ -2810,6 +2856,8 @@ void ComputeHessianQuality::initFromFrontend(omxState *globalState, SEXP rObj)
 
 void ComputeHessianQuality::reportResults(FitContext *fc, MxRList *slots, MxRList *)
 {
+	if (!(fc->wanted & (FF_COMPUTE_HESSIAN | FF_COMPUTE_IHESSIAN))) return;
+
 	// See Luenberger & Ye (2008) Second Order Test (p. 190) and Condition Number (p. 239)
 
 	if (fc->infoDefinite != NA_LOGICAL || !doubleEQ(fc->infoCondNum, NA_REAL)) {
@@ -2906,3 +2954,224 @@ void ComputeReportExpectation::reportResults(FitContext *fc, MxRList *, MxRList
 
 	result->add("expectations", expectations);
 }
+
+ComputeBootstrap::~ComputeBootstrap()
+{
+	if (plan) delete plan;
+}
+
+void ComputeBootstrap::initFromFrontend(omxState *globalState, SEXP rObj)
+{
+	super::initFromFrontend(globalState, rObj);
+
+	SEXP slotValue;
+	SEXP s4class;
+
+	Rf_protect(slotValue = R_do_slot(rObj, Rf_install("plan")));
+	Rf_protect(s4class = STRING_ELT(Rf_getAttrib(slotValue, R_ClassSymbol), 0));
+	plan = omxNewCompute(globalState, CHAR(s4class));
+	plan->initFromFrontend(globalState, slotValue);
+
+	ProtectedSEXP Rdata(R_do_slot(rObj, Rf_install("data")));
+	for (int wx=0; wx < Rf_length(Rdata); ++wx) {
+		if (isErrorRaised()) return;
+		int objNum = INTEGER(Rdata)[wx];
+		context ctx;
+		ctx.data = globalState->dataList[objNum];
+		int numRows = ctx.data->numRawRows();
+		if (!numRows) {
+			Rf_error("%s: '%s' cannot have row weights",
+				 name, ctx.data->name);
+		}
+		ctx.origRowWeights = ctx.data->getWeightColumn();
+		ctx.origCumSum.resize(numRows);
+		ctx.resample.resize(ctx.origCumSum.size());
+		if (ctx.origRowWeights) {
+			std::partial_sum(ctx.origRowWeights, ctx.origRowWeights + ctx.origCumSum.size(),
+					 ctx.origCumSum.begin());
+		} else {
+			for (int rx=0; rx < numRows; ++rx) ctx.origCumSum[rx] = 1+rx;
+		}
+		contexts.push_back(ctx);
+	}
+
+	ProtectedSEXP Rverbose(R_do_slot(rObj, Rf_install("verbose")));
+	verbose = Rf_asInteger(Rverbose);
+
+	ProtectedSEXP Rrepl(R_do_slot(rObj, Rf_install("replications")));
+	numReplications = Rf_asInteger(Rrepl);
+
+	ProtectedSEXP Rparallel(R_do_slot(rObj, Rf_install("parallel")));
+	parallel = Rf_asLogical(Rparallel);
+
+	ProtectedSEXP Ronly(R_do_slot(rObj, Rf_install("only")));
+	only = Rf_asInteger(Ronly);
+	if (only != NA_INTEGER) {
+		numReplications = 1;
+	}
+
+	previousNumParam = -1;
+	previousData = 0;
+
+	ProtectedSEXP Routput(R_do_slot(rObj, Rf_install("output")));
+	ProtectedSEXP RoutputNames(Rf_getAttrib(Routput, R_NamesSymbol));
+	for (int ax=0; ax < Rf_length(Routput); ++ax) {
+		const char *key = R_CHAR(STRING_ELT(RoutputNames, ax));
+		SEXP val = VECTOR_ELT(Routput, ax);
+		if (strEQ(key, "raw")) {
+			previousData = val;
+		} else if (strEQ(key, "numParam")) {
+			previousNumParam = Rf_asInteger(val);
+		}
+	}
+}
+
+void ComputeBootstrap::computeImpl(FitContext *fc)
+{
+	if (verbose >= 1) mxLog("%s: %d replications seed=%d parallel=%d",
+				name, numReplications, seed, int(parallel));
+
+	int numCols = fc->numParam + 3;
+	Rf_protect(rawOutput = Rf_allocVector(VECSXP, numCols));
+	SEXP colNames = Rf_allocVector(STRSXP, numCols);
+	Rf_setAttrib(rawOutput, R_NamesSymbol, colNames);
+
+	SET_STRING_ELT(colNames, 0, Rf_mkChar("seed"));
+	SET_VECTOR_ELT(rawOutput, 0, Rf_allocVector(INTSXP, numReplications));
+	SET_STRING_ELT(colNames, 1, Rf_mkChar("fit"));
+	SET_VECTOR_ELT(rawOutput, 1, Rf_allocVector(REALSXP, numReplications));
+	for (int px=0; px < int(fc->numParam); ++px) {
+		SET_STRING_ELT(colNames, 2+px, Rf_mkChar(varGroup->vars[px]->name));
+		SET_VECTOR_ELT(rawOutput, 2+px, Rf_allocVector(REALSXP, numReplications));
+	}
+	SET_STRING_ELT(colNames, 2+fc->numParam, Rf_mkChar("statusCode"));
+	SET_VECTOR_ELT(rawOutput, 2+fc->numParam, allocInformVector(numReplications));
+	markAsDataFrame(rawOutput, numReplications);
+
+	if (previousData && (previousNumParam != int(fc->numParam) ||
+			     Rf_length(previousData) != Rf_length(rawOutput))) {
+		if (verbose >= 1) mxLog("%s: discarded mismatching previous data (%d/%d %d/%d)",
+					name, previousNumParam, int(fc->numParam),
+					Rf_length(previousData), Rf_length(rawOutput));
+		previousData = 0;
+	}
+
+	for (int repl=0; repl < numReplications; ++repl) {
+		INTEGER(VECTOR_ELT(rawOutput, 0))[repl] = NA_INTEGER;
+		for (int cx=0; cx <= int(fc->numParam); ++cx) {
+			REAL(VECTOR_ELT(rawOutput, 1 + cx))[repl] = NA_REAL;
+		}
+		INTEGER(VECTOR_ELT(rawOutput, 2 + fc->numParam))[repl] = NA_INTEGER;
+	}
+	if (only == NA_INTEGER && previousData) {
+		int toCopy = std::min(Rf_length(VECTOR_ELT(previousData, 0)),
+				      numReplications);
+		if (verbose >= 1) mxLog("%s: copying %d rows from previous run", name, toCopy);
+		memcpy(INTEGER(VECTOR_ELT(rawOutput, 0)),
+		       INTEGER(VECTOR_ELT(previousData, 0)),
+		       toCopy * sizeof(int));
+		for (int cx=0; cx <= int(fc->numParam); ++cx) {
+			memcpy(REAL(VECTOR_ELT(rawOutput, 1+cx)),
+			       REAL(VECTOR_ELT(previousData, 1+cx)),
+			       toCopy * sizeof(double));
+		}
+		memcpy(INTEGER(VECTOR_ELT(rawOutput, 2 + fc->numParam)),
+		       INTEGER(VECTOR_ELT(previousData, 2 + fc->numParam)),
+		       toCopy * sizeof(int));
+	}
+
+	// implement parallel TODO
+
+	auto *seedVec = INTEGER(VECTOR_ELT(rawOutput, 0));
+	if (only == NA_INTEGER || !previousData) {
+		GetRNGstate();
+		for (int repl=0; repl < numReplications; ++repl) {
+			if (seedVec[repl] != NA_INTEGER) continue;
+			int seed1 = unif_rand() * std::numeric_limits<int>::max();
+			if (seed1 == NA_INTEGER) seed1 = 0; // maybe impossible
+			seedVec[repl] = seed1;
+		}
+		PutRNGstate();
+	} else {
+		if (only <= Rf_length(VECTOR_ELT(previousData, 0))) {
+			if (verbose >= 1) mxLog("%s: using only=%d", name, only);
+			seedVec[0] = INTEGER(VECTOR_ELT(previousData, 0))[only - 1];
+		} else {
+			Rf_error("%s: only=%d but previous data has just %d replications",
+				 name, only, Rf_length(VECTOR_ELT(previousData, 0)));
+		}
+	}
+
+	Eigen::VectorXd origEst = fc->getEst();
+
+	for (int repl=0; repl < numReplications && !isErrorRaised(); ++repl) {
+		std::mt19937 generator(seedVec[repl]);
+		if (INTEGER(VECTOR_ELT(rawOutput, 2 + fc->numParam))[repl] != NA_INTEGER) continue;
+		if (verbose >= 2) mxLog("%s: replication %d", name, repl);
+		for (auto &ctx : contexts) {
+			ctx.resample.assign(ctx.origCumSum.size(), 0.0);
+			int last = ctx.origCumSum.size() - 1;
+			int total = ctx.origCumSum[last];
+			std::uniform_int_distribution<int> dist(1, total);
+			for (int sx=0; sx < total; ++sx) {
+				int pick = dist(generator);
+				auto rowPick = std::lower_bound(ctx.origCumSum.begin(), ctx.origCumSum.end(), pick);
+				int row = rowPick - ctx.origCumSum.begin();
+				ctx.resample[row] += 1.0;
+			}
+			if (verbose >= 4) {
+				EigenStdVectorAdaptor<double> rs(ctx.resample);
+				mxPrintMat(ctx.data->name, rs);
+			}
+			ctx.data->setWeightColumn(ctx.resample.data());
+			if (only != NA_INTEGER) {
+				onlyWeight.add(ctx.data->name, Rcpp::wrap(ctx.resample));
+			}
+		}
+		fc->state->invalidateCache();
+		fc->getEst() = origEst;
+		plan->compute(fc);
+		if (only == NA_INTEGER) {
+			fc->wanted &= ~FF_COMPUTE_DERIV;  // discard garbage
+		}
+		if (verbose >= 3) {
+			auto est = fc->getEst();
+			mxPrintMat("est", est);
+		}
+		REAL(VECTOR_ELT(rawOutput, 1))[repl] = fc->fit;
+		for (int px=0; px < int(fc->numParam); ++px) {
+			REAL(VECTOR_ELT(rawOutput, 2 + px))[repl] = fc->est[px];
+		}
+		INTEGER(VECTOR_ELT(rawOutput, 2 + fc->numParam))[repl] = fc->wrapInform();
+		reportProgress(fc);
+	}
+
+	for (auto &ctx : contexts) {
+		ctx.data->setWeightColumn(ctx.origRowWeights);
+	}
+
+	if (only == NA_INTEGER) {
+		fc->setInform(INFORM_UNINITIALIZED);
+		fc->getEst() = origEst;
+		fc->copyParamToModel();
+	}
+}
+
+void ComputeBootstrap::collectResults(FitContext *fc, LocalComputeResult *lcr, MxRList *out)
+{
+	super::collectResults(fc, lcr, out);
+	std::vector< omxCompute* > clist(1);
+	clist[0] = plan;
+	collectResultsHelper(fc, clist, lcr, out);
+}
+
+void ComputeBootstrap::reportResults(FitContext *fc, MxRList *slots, MxRList *)
+{
+	MxRList output;
+	output.add("numParam", Rcpp::wrap(int(fc->numParam)));
+	output.add("raw", rawOutput);
+	if (only != NA_INTEGER) {
+		output.add("weight", onlyWeight.asR());
+	}
+	slots->add("output", output.asR());
+}
