@@ -15,12 +15,10 @@
  */
 
 #include "omxFitFunction.h"
+#include "Compute.h"
+#include "EnableWarnings.h"
 
-#ifdef SHADOW_DIAG
-#pragma GCC diagnostic warning "-Wshadow"
-#endif
-
-struct ssMLFitState {
+struct ssMLFitState : omxFitFunction {
 	bool returnRowLikelihoods;
 	bool populateRowDiagnostics;
 	omxMatrix *cov;
@@ -28,11 +26,16 @@ struct ssMLFitState {
 	omxMatrix *contRow;
 	omxMatrix *rowLikelihoods;
 	omxMatrix *RCX;
+
+	virtual ~ssMLFitState();
+	virtual void init();
+	virtual void compute(int ffcompute, FitContext *fc);
+	virtual void populateAttr(SEXP algebra);
 };
 
-void populate(omxFitFunction *off, SEXP algebra)
+void ssMLFitState::populateAttr(SEXP algebra)
 {
-	ssMLFitState *argStruct = ((ssMLFitState*)off->argStruct);
+	ssMLFitState *argStruct = this;
 
 	if(argStruct->populateRowDiagnostics){
 		SEXP rowLikelihoodsExt;
@@ -44,20 +47,15 @@ void populate(omxFitFunction *off, SEXP algebra)
 	}
 }
 
-static void compute(omxFitFunction *oo, int want, FitContext *fc)
+void ssMLFitState::compute(int want, FitContext *fc)
 {
 	if (want & (FF_COMPUTE_INITIAL_FIT | FF_COMPUTE_PREOPTIMIZE)) return;
 
-	ssMLFitState *state = (ssMLFitState *) oo->argStruct;
-	omxExpectation *expectation = oo->expectation;
+	auto *oo = this;
+	ssMLFitState *state = this;
 	auto dataColumns	= expectation->getDataColumns();
 	omxData *data = expectation->data;
 	int rowcount = data->rows;
-	omxMatrix *smallRow = state->smallRow;
-	omxMatrix *contRow = state->contRow;
-	omxMatrix *cov = state->cov;
-	omxMatrix *RCX = state->RCX;
-	omxMatrix *rowLikelihoods = state->rowLikelihoods;
 
 	omxSetExpectationComponent(expectation, "Reset", NULL);
 	Eigen::VectorXi contRemove(cov->cols);
@@ -82,7 +80,10 @@ static void compute(omxFitFunction *oo, int want, FitContext *fc)
 				++numCont;
 			}
 		}
-		if (!numCont) continue;
+		if (!numCont) {
+			omxSetMatrixElement(rowLikelihoods, row, 0, 1.0);
+			continue;
+		}
 
 		omxMatrix *smallMeans = omxGetExpectationComponent(expectation, "means");
 		omxRemoveElements(smallMeans, contRemove.data());
@@ -91,9 +92,18 @@ static void compute(omxFitFunction *oo, int want, FitContext *fc)
 		//Get covInfo from state space expectation
 		int info = (int) omxGetExpectationComponent(expectation, "covInfo")->data[0];
 		if(info!=0) {
-			if (fc) fc->recordIterationError("Expected covariance matrix is "
-							 "not positive-definite in data row %d",
-							 row);
+			EigenMatrixAdaptor Ecov(smallCov);
+			if (Ecov.rows() > 50) {
+				if (fc) fc->recordIterationError("%s: expected covariance matrix is "
+								 "not positive-definite in data row %d",
+								 oo->name(), row);
+			} else {
+				std::string empty = std::string("");
+				std::string buf = mxStringifyMatrix("covariance", Ecov, empty);
+				if (fc) fc->recordIterationError("%s: expected covariance matrix is "
+								 "not positive-definite in data row %d; Details:\n%s",
+								 oo->name(), row, buf.c_str());
+			}
 			omxSetMatrixElement(oo->matrix, 0, 0, NA_REAL);
 			return;
 		}
@@ -128,9 +138,9 @@ static void compute(omxFitFunction *oo, int want, FitContext *fc)
 		// floating-point addition is not associative,
 		// so we serialized the following reduction operation.
 		for(int i = 0; i < data->rows; i++) {
-			double val = log(omxVectorElement(state->rowLikelihoods, i));
-			//mxLog("[%d] %g", i, -2.0 * val);
-			sum += val;
+			double prob = omxVectorElement(state->rowLikelihoods, i);
+			//mxLog("[%d] %g", i, -2.0 * log(prob));
+			sum += log(prob);
 		}	
 		if(OMX_DEBUG) {mxLog("%s: total likelihood is %3.3f", oo->name(), sum);}
 		omxSetMatrixElement(oo->matrix, 0, 0, -2.0 * sum);
@@ -139,29 +149,29 @@ static void compute(omxFitFunction *oo, int want, FitContext *fc)
 	}
 }
 
-static void destroy(omxFitFunction *oo)
+ssMLFitState::~ssMLFitState()
 {
-	ssMLFitState *state = (ssMLFitState *) oo->argStruct;
+	ssMLFitState *state = this;
 	omxFreeMatrix(state->smallRow);
 	omxFreeMatrix(state->contRow);
 	omxFreeMatrix(state->rowLikelihoods);
-	delete state;
 }
 
-void ssMLFitInit(omxFitFunction* oo)
+omxFitFunction *ssMLFitInit()
+{ return new ssMLFitState; }
+
+void ssMLFitState::init()
 {
-	ssMLFitState *state = new ssMLFitState;
-	oo->argStruct = state;
-	oo->computeFun = compute;
-	oo->destructFun = destroy;
-	oo->populateAttrFun = populate;
+	auto *oo = this;
+	auto *state = this;
+
 	oo->openmpUser = false;
 	oo->canDuplicate = true;
 
 	state->returnRowLikelihoods = Rf_asInteger(R_do_slot(oo->rObj, Rf_install("vector")));
-	state->populateRowDiagnostics = Rf_asInteger(R_do_slot(oo->rObj, Rf_install("rowDiagnostics")));
+	units = returnRowLikelihoods? FIT_UNITS_PROBABILITY : FIT_UNITS_MINUS2LL;
 
-	omxExpectation *expectation = oo->expectation;
+	state->populateRowDiagnostics = Rf_asInteger(R_do_slot(oo->rObj, Rf_install("rowDiagnostics")));
 
 	omxState *currentState = oo->matrix->currentState;
 	state->rowLikelihoods = omxInitMatrix(expectation->data->rows, 1, TRUE, currentState);
